import pytest

from zigzag.api import get_hardware_performance_zigzag

workloads = (
    "zigzag/inputs/workload/resnet18.onnx",
    "zigzag/inputs/workload/resnet18.yaml",
)

# Expected energy and latency for each workload defined above
ens_lats = {
    "zigzag/inputs/workload/alexnet.onnx": (6039993369, 8373281),
    "zigzag/inputs/workload/mobilenetv2.onnx": (930699349, 1955657),
    "zigzag/inputs/workload/resnet18.onnx": (1725111410.78, 2709223.75),
    "zigzag/inputs/workload/resnet18.yaml": (2213422347.76, 3296500.58),
}


@pytest.fixture
def mapping():
    return "zigzag/inputs/mapping/tesla_npu_like.yaml"


@pytest.fixture
def accelerator():
    return "zigzag/inputs/hardware/tesla_npu_like.yaml"


@pytest.mark.parametrize("workload", workloads)
def test_api(workload: str, accelerator: str, mapping: str):  # pylint: disable=W0621
    energy, latency, _ = get_hardware_performance_zigzag(workload, accelerator, mapping, exploit_data_locality=True)
    (expected_energy, expected_latency) = ens_lats[workload]
    print(f"'{workload}': ({energy}, {latency}),")
    assert energy == pytest.approx(expected_energy)  # type: ignore
    assert latency == pytest.approx(expected_latency)  # type: ignore
